#!/usr/bin/python

# $Id: rst2latex.py 5905 2009-04-16 12:04:49Z milde $
# Author: David Goodger <goodger@python.org>
# Copyright: This module has been placed in the public domain.

"""
A minimal front end to the Docutils Publisher, producing LaTeX.
"""

from docutils.writers.latex2e import Writer, LaTeXTranslator, Babel, Table, PreambleCmds

from docutils import nodes
from docutils.parsers.rst import directives
from docutils.parsers.rst import Directive

class ModernTable(Table):

    def __init__(self,translator,latex_type,table_style):
        Table.__init__(self, translator, latex_type, table_style)

        PreambleCmds.table = r"""\usepackage{longtable,array}
        \setlength{\extrarowheight}{2pt}
        \newlength{\DUtablewidth} % internal use in tables"""

    def get_latex_type(self):
        return('longtable')


class ModernCVTranslator(LaTeXTranslator):
    """
    This is a translator class for the docutils system.
    It will produce a minimal set of html output.
    (No extry divs, classes oder ids.)

    """
    class_params = (
            'moderncvstyle',
            'moderncvcolor',
            'mobile',
            'fax',
            'email',
            'homepage',
            'extrainfo',
            'address',

            'opening',
            'closing',

        )

    def __init__(self, document, babel_class=Babel):
        LaTeXTranslator.__init__(self, document, babel_class)
        self.active_table = ModernTable(self, 'longtable', self.settings.table_style)

        # Set when processing a moderncv document option
        self.cventry = False
        # Set when inside a cventry
        self.in_definition = False

        self.entry_level = 0

    def depart_document(self, node):
        # Complete header with information gained from walkabout
        # * language setup
        if (self.babel.otherlanguages or
            self.babel.language not in ('', 'english')):
            self.requirements['babel'] = self.babel()
        # * conditional requirements (before style sheet)
        self.requirements = self.requirements.sortedvalues()
        # * coditional fallback definitions (after style sheet)
        self.fallbacks = self.fallbacks.sortedvalues()
        # Complete body
        # * document title (with "use_latex_docinfo" also
        #   'author', 'organization', 'contact', 'address' and 'date')

        if self.date:
            self.titledata.append(r'\date{%s}' % ', '.join(self.date))

        if self.title:
            # with the default template, titledata is written to the preamble
            self.titledata.append('%%% Title Data')
            # \title (empty \title prevents error with \maketitle)
            title = [''.join(self.title)] #+ self.title_labels
            if self.subtitle:
                title += [r'\\ % subtitle',
                          r'\DUdocumentsubtitle{%s}' % ''.join(self.subtitle)
                         ] + self.subtitle_labels
            self.titledata.append(r'\title{%s}' % '%\n  '.join(title))
            # \date (empty \date prevents defaulting to \today)
            # self.titledata.append(r'\date{%s}' % ', '.join(self.date))
            # \maketitle in the body formats title with LaTeX
            self.body_pre_docinfo.append('\\maketitle\n')

        # * bibliography
        #   TODO insertion point of bibliography should be configurable.
        if self._use_latex_citations and len(self._bibitems)>0:
            if not self.bibtex:
                widest_label = ''
                for bi in self._bibitems:
                    if len(widest_label)<len(bi[0]):
                        widest_label = bi[0]
                self.out.append('\n\\begin{thebibliography}{%s}\n' %
                                 widest_label)
                for bi in self._bibitems:
                    # cite_key: underscores must not be escaped
                    cite_key = bi[0].replace(r'\_','_')
                    self.out.append('\\bibitem[%s]{%s}{%s}\n' %
                                     (bi[0], cite_key, bi[1]))
                self.out.append('\\end{thebibliography}\n')
            else:
                self.out.append('\n\\bibliographystyle{%s}\n' %
                                self.bibtex[0])
                self.out.append('\\bibliography{%s}\n' % self.bibtex[1])
        # * make sure to generate a toc file if needed for local contents:
        if 'minitoc' in self.requirements and not self.has_latex_toc:
            self.out.append('\n\\faketableofcontents % for local ToCs\n')

    def visit_author(self, node):
        self.titledata.append(r'\name{%s}' % node.astext().replace(" ", "}{"))
        raise nodes.SkipNode

    def visit_address(self, node):
        self.titledata.append(r'\address{%s}' % node.astext().replace("\n", "}{"))
        raise nodes.SkipNode

    def visit_field_name(self, node):
        if self.out is self.docinfo:
            text = node.astext()
            if text in self.class_params:
                self.cventry = True
                self.titledata.append(r'\%s' % text)
                raise nodes.SkipNode
            else:
                self.cventry = False
                LaTeXTranslator.visit_field_name(self, node)
        else:
            # Commands with optional args inside an optional arg must be put
            # in a group, e.g. ``\item[{\hyperref[label]{text}}]``.
            self.out.append('\\cvitem{')

    def depart_field_name(self, node):
        if self.out is not self.docinfo:
            self.out.append('}')
        elif not self.cventry:
            LaTeXTranslator.depart_field_name(self, node)

    def depart_field(self, node):
        if self.out is self.docinfo:
            self.out.pop()
        else:
            self.out.append('\n')

    def visit_field_body(self, node):
        if self.out is not self.docinfo:
            self.out.append("{%")
            self.in_definition = True
        elif self.cventry:
                self.titledata.append(r'{%s}' % node.astext())
                self.out.append(r'\\')
                raise nodes.SkipNode
        else:
            LaTeXTranslator.visit_field_body(self, node)

    def depart_field_body(self, node):
        if self.out is not self.docinfo:
            # Remove the last element added by depart_paragraph
            if self.entry_level == 1:
                self.out.pop()
            self.out.append("}")
            self.in_definition = False
        elif self.cventry:
            self.cventry = False
        else:
            LaTeXTranslator.depart_field_body(self, node)

    def visit_bullet_list(self, node):
        if self.entry_level == 1:
            self.out.pop()
        self.entry_level += 1
        if self.is_toc_list:
            self.out.append( '%\n\\begin{list}{}{}\n' )
        else:
            self.out.append( '%\n\\begin{itemize}' )

    def depart_bullet_list(self, node):
        self.entry_level -= 1
        if self.is_toc_list:
            self.out.append( '\n\\end{list}\n' )
        else:
            self.out.append( '\n\\end{itemize}')
        if self.entry_level == 1:
            self.out.append("")

    def visit_definition_list(self, node):
        self.out.append( '%\n\\begin{description}' )

    def depart_definition_list(self, node):
        self.out.append( '\\end{description}' )

    def visit_field_list(self, node):
        self.entry_level += 1

    def depart_field_list(self, node):
        self.entry_level -= 1

    def depart_paragraph(self, node):
        if not self.in_definition:
            LaTeXTranslator.depart_paragraph(self, node)
        else:
            if self.entry_level <= 1:
                self.out.append("\\newline")

class Close(Directive):
    required_arguments = 1
    optional_arguments = 0
    has_content = False

    def run(self):
        text = "\closing{%s}\n\makeletterclosing\n" % self.arguments[0]
        return [nodes.raw('', text, format="latex")]

class Open(Directive):
    required_arguments = 1
    optional_arguments = 0

    def build_option(format):
      return lambda x: format % ( directives.unchanged_required(x.replace("\n", "\\\\")) )

    def get_option(self, key, default):
        try:
            return self.options[key]
        except:
            return default

    option_spec = {
        "to": build_option("%s"),
        "address": build_option("%s"),
        }

    def run(self):
        text = """\\recipient{%s}{%s}
        \opening{%s}
        \makelettertitle""" % (
            self.get_option("to", "\\ "),
            self.get_option("address", ""),
            self.arguments[0]
        )
        return [nodes.raw('', text, format="latex")]



def configureWriter(writer):
    directives.register_directive('close', Close)
    directives.register_directive('open', Open)
    writer.translator_class = ModernCVTranslator

